/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file AuthorityDescription.h
 *    Descriptive informations about an authority.
 *
 * @author Erik Einhorn, Tim Langner
 * @date   2010/10/04
 */

#ifndef _MIRA_AUTHORITYDESCRIPTION_H_
#define _MIRA_AUTHORITYDESCRIPTION_H_

#include <fw/ResourceName.h>

namespace mira {

///////////////////////////////////////////////////////////////////////////////

/**
 * @ingroup FWModule
 * This is the descriptive part of an authority. It stores name, id and
 * namespace.
 */
class MIRA_FRAMEWORK_EXPORT AuthorityDescription
{
public:

	template<typename Reflector>
	void reflect(Reflector& r)
	{
		r.member("Namespace", ns, "Namespace");
		r.member("ID", id, "The ID (unique in the namespace)");
		r.member("Name", name, "The name");
	}

public:

	/**
	 * Return the fully qualified global name (includes namespace)
	 * This method returns the name without the trailing UUID added
	 * if the authority is anonymous.
	 * @return Global Name
	 */
	std::string getGlobalName() const {
		return ns / name;
	}

	/**
	 * Return the fully qualified global id (includes namespace)
	 * @return Global ID
	 */
	std::string getGlobalID() const {
		return ns / id;
	}

public:

	ResourceName ns; 	///< the full namespace (starting from root) of this authority
	std::string id;  	///< the authority id. will be generated for anonymous authorities
	std::string name;	///< the name of the authority (equals id for non anonymous authorities)
};

///////////////////////////////////////////////////////////////////////////////

}

#endif
