/*
 * Copyright (C) by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file MetaTypeDatabaseView.h
 *    Show entries in the MIRA framework's Type Information Database
 *
 * @author Christof Schröter
 * @date   2019/03/17
 */

#ifndef _MIRA_METATYPEDATABASEVIEW_H_
#define _MIRA_METATYPEDATABASEVIEW_H_

#include <rcp/EditorPart.h>

#include <QTreeWidget>
#include <QTimerEvent>

#include <serialization/MetaSerializer.h>

namespace mira {

///////////////////////////////////////////////////////////////////////////////

class MetaTypeDatabaseView : public mira::EditorPart
{
	Q_OBJECT;
	MIRA_META_OBJECT(MetaTypeDatabaseView,
                ("Category", "Information")
                ("Name", "MetaType Database")
                ("Description", "Shows entries in the MetaType database."))

public:
	MetaTypeDatabaseView();
	virtual ~MetaTypeDatabaseView()  {}

protected:
	virtual QWidget* createPartControl();
	virtual void timerEvent(QTimerEvent* event);

private:
	void updateDB();

	QTreeWidgetItem* addTypeInfoCategory(const QString& category);

	void showMetaType(CompoundMetaPtr meta);

	void showVersions(CompoundMetaPtr meta);
	void showMembers(CompoundMetaPtr meta);
	void showInterfaces(CompoundMetaPtr meta);
	void showMethods(CompoundMetaPtr meta);

private slots:
	void onSelect();
	
private:
	QWidget* mViewWidget;
	QTreeWidget* mTypesListView;
	QTreeWidget* mTypeInfoView;
	int mUpdateTimer;
};

///////////////////////////////////////////////////////////////////////////////

}

#endif
